# frozen_string_literal: true

SERVICE_NAME = 'user_management'
SERVICE_TASK = 'webserver'

require 'bundler/setup'
require_relative 'system/import'

require 'user_management/graphql'

require 'sinatra/instrumentation'
require 'sinatra/json'
require 'sinatra/custom_logger'

require 'rollbar/middleware/sinatra'
require 'cht-lib-fabric/helpers/context'
require 'cht-lib-fabric/helpers/graphql_query'
require 'cht-lib-fabric/helpers/puma_stats'
require 'cht-lib-fabric/helpers/tokens'
require 'cht-lib-fabric/helpers/web_server'
require 'cht-lib-fabric/middleware/statsd'

require 'user_management/actions/update_user_idp'

Application.finalize!

##
# Main entry point to service.
# This is where you will define your routes and endpoints.
class WebServer < ::Sinatra::Base
  include Import[:tracer, 'graphql.tracers']

  register ::Platform::Helpers::GraphQLQuery
  helpers ::Platform::Helpers::WebServer
  helpers Sinatra::CustomLogger
  register ::Platform::Helpers::Context
  register ::Platform::Helpers::PumaStats
  register ::Platform::Helpers::Tokens

  configure do
    disable :method_override
    disable :static

    set :logger, Application['logger']
    enable :logging
    set :protection, except: :session_hijacking
    set :root, File.dirname(__FILE__)

    set :cht_pod, Application[:settings].pod_name
    set :jwks_uri, Application[:settings].jwks_uri
    set :wavefront, Application[:wavefront]

    if Application[:settings].developer_jwt_signing != '' ||
       (Application.env != :production &&
        Application[:settings].jwks_uri == '')
      enable :jwt_developer_key
    end
  end

  use ::Rollbar::Middleware::Sinatra
  use ::Rack::Deflater
  use ::Platform::Middleware::StatsD, Application['statsd']
  register ::Sinatra::Instrumentation
  
  get '/' do
    "#{SERVICE_NAME} - #{SERVICE_TASK}"
  end

  graphql '/graphql', ::UserManagement::GraphQL::Schema

  post '/updateUserIdp' do
    begin
      # Make sure that a cloudpercept service account is accessing this
      caller_validator = ::UserManagement::Utils::CallerValidator.new(request, token_validator)
      halt(401, {message: 'Unauthorized to make this request' }.to_json) unless caller_validator.cloudpercept_service_account?
      body = JSON.parse(request.body.read)
      halt(400, {message: 'Request Parameters missing. Requires "customer_id", "user_id", and "raw_info"'}) if [body['customer_id'], body["user_id"], body["raw_info"]].any?(&:nil?)
      response = ::UserManagement::Actions::UpdateIdentityAttributes.new.call(body['customer_id'], body["user_id"], body["raw_info"])
      status 200
      body response.to_json
    rescue => ex
      logger.error(ex.backtrace)
      halt(500, { message: 'Something went wrong. Investigate the Logs' }.to_json)
    end
  end
end
